import { StatsModule } from "./StatsModule";
import { EnabledStats, StatsManagerOptions, StatsAcquisition, SavedStatsFormat } from "./types/types";
/**
 * Creates a new StatsManager instance
 * @class StatsManager
 * @description Manages the stats of the bot.
 */
declare abstract class StatsManager {
    saveInterval: number;
    enabledStats: EnabledStats;
    private _statsModules;
    /**
     * Manages the stats of your discord bot.
     * @param {StatsManagerOptions} options - The options for the stats manager.
     * @param {number} [options.saveInterval = 3_600_000] - The interval in milliseconds default 1 hour.
     * @param {EnabledStats} [options.enabledStats = {}] - The enabled stats.
     */
    constructor(options: StatsManagerOptions);
    /**
     * Initializes the stats manager.
     * @private
     * @returns {void}
     */
    _init(): void;
    /**
     * Returns the enabled stats modules.
     * @returns {StatsModule}
     */
    get statsModules(): StatsModule[];
    /**
     * Returns the targetted stats module.
     * @param {string} name - The name of the stats module.
     * @returns {StatsModule|undefined}
     */
    findModule(name: string): StatsModule;
    /**
     * Acquires the stats.
     * @param {StatsAcquisition} returnedStatsFunction - The function that returns the stats.
     * @returns {Promise<void>}
     */
    start(returnedStatsFunction: StatsAcquisition): Promise<void>;
    /**
     * Saves the stats.
     * @abstract
     * @param {SavedStatsFormat} stats - The stats to save.
     */
    abstract saveStats(stats: SavedStatsFormat): Promise<void>;
    /**
     * Gets the stats.
     * @abstract
     * @returns {Promise<SavedStatsFormat[]>}
     */
    abstract getStats(): Promise<SavedStatsFormat[]>;
    /**
     * Deletes the stats.
     * @abstract
     * @param {number[]} timestamps - The timestamps of the stats to delete.
     */
    abstract deleteStats(timestamps: number[]): Promise<void>;
}
export { StatsManager };
